"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watch = exports.build = exports.preBuild = void 0;
const chokidar_1 = __importDefault(require("chokidar"));
const live_server_1 = __importDefault(require("live-server"));
const config_1 = require("./config");
const transform_1 = require("./transform");
const ws_1 = __importDefault(require("ws"));
const http_1 = __importDefault(require("http"));
const browserHtml_1 = require("./browserHtml");
const browser_1 = require("./browser");
const net_1 = require("net");
const fs_1 = __importDefault(require("fs"));
const dotenv_1 = require("dotenv");
const graphql_zeus_1 = require("graphql-zeus");
const fsAddons_1 = require("./fsAddons");
const transpilerConfig_1 = require("./transpilerConfig");
const paths_1 = require("./paths");
const transformHeaders = (headers) => {
    if (!headers) {
        return undefined;
    }
    return Object.keys(headers).map((k) => `${k}:${headers[k]}`);
};
const envs = () => fs_1.default.existsSync('./.env') ? (0, dotenv_1.parse)(fs_1.default.readFileSync('./.env')) : {};
const internals = (config, build) => {
    const ssg = {
        env: envs(),
        config,
        host: build ? `http://0.0.0.0:${config.port}` : '.',
    };
    return `const ssg = ${JSON.stringify(ssg, null, 4)}`;
};
const getPort = (port = 80, maxPort = 65535) => {
    if (maxPort < port) {
        return Promise.reject((() => {
            const e = new Error('EPORTSPEC');
            return e;
        })());
    }
    const server = (0, net_1.createServer)();
    return new Promise((resolve, reject) => server
        .once('error', (error) => ++port > maxPort ? reject(error) : server.listen(port))
        .once('listening', () => server.close(() => resolve(port)))
        .listen(port));
};
const initBrowserBundler = ({ config }) => __awaiter(void 0, void 0, void 0, function* () {
    const ws = new ws_1.default.Server({
        port: config.websocketPort,
        perMessageDeflate: true,
    });
    ws.on('connection', (client) => {
        client.send(JSON.stringify({ type: 'connected' }));
        client.on('message', (e) => {
            ws.clients.forEach((v) => {
                v.send(e);
            });
        });
    });
    const browserBundle = http_1.default.createServer((req, res) => {
        const requestURL = req.url;
        if (requestURL === '/' || !requestURL) {
            res.writeHead(200, { 'content-type': 'text/html' });
            res.write((0, browserHtml_1.browserHtml)(config));
        }
        else {
            const [, , ...fileNames] = requestURL.split('/');
            const fileName = fileNames.join('/');
            if ((fileName === null || fileName === void 0 ? void 0 : fileName.endsWith('.js')) || (fileName === null || fileName === void 0 ? void 0 : fileName.endsWith('.mjs'))) {
                const filePath = (0, paths_1.pathIn)(config)(fileName);
                const pathContent = fs_1.default.readFileSync(filePath).toString('utf-8');
                const fContent = [internals(config), pathContent].join('\n');
                res.writeHead(200, { 'content-type': 'text/javascript' });
                res.write(fContent);
            }
        }
        res.end();
    });
    const browserBundlePort = yield getPort(config.port + 1);
    browserBundle.listen(browserBundlePort);
    const browser = yield (0, browser_1.runBrowser)(browserBundlePort);
    return {
        browserBundle,
        ws,
        browser,
        close: () => __awaiter(void 0, void 0, void 0, function* () {
            ws.close();
            browserBundle.close();
            yield browser.close();
        }),
    };
});
const preBuild = (config) => __awaiter(void 0, void 0, void 0, function* () {
    (0, transform_1.cleanBuild)(config);
    if (config.mode) {
        (0, transpilerConfig_1.regenerateTsConfig)(config);
    }
    else {
        (0, transpilerConfig_1.regenerateJsConfig)(config);
    }
    yield (0, transform_1.generateBasicTypingsFiles)(config);
    if (config.graphql) {
        yield Promise.all(Object.keys(config.graphql).map((k) => __awaiter(void 0, void 0, void 0, function* () {
            const schema = yield graphql_zeus_1.Utils.getFromUrl(config.graphql[k].url, transformHeaders(config.graphql[k].headers));
            yield (0, transform_1.generateTypingsFiles)({ config, schema, name: k });
            return {
                schema,
                name: k,
            };
        })));
    }
});
exports.preBuild = preBuild;
const build = () => __awaiter(void 0, void 0, void 0, function* () {
    const config = (0, config_1.readConfig)('./purplehaze.json');
    yield (0, exports.preBuild)(config);
    const { close } = yield initBrowserBundler({
        config,
    });
    yield (0, transform_1.transformFiles)({ config });
    (0, transform_1.copyStaticFiles)(config);
    yield close();
});
exports.build = build;
const watch = () => __awaiter(void 0, void 0, void 0, function* () {
    let block = true;
    let liveServerRunning = false;
    const config = (0, config_1.readConfig)('./purplehaze.json');
    yield (0, exports.preBuild)(config);
    (0, transform_1.copyStaticFiles)(config);
    yield initBrowserBundler({
        config,
    });
    chokidar_1.default
        .watch((0, paths_1.pathIn)(config)(`**/*.{js,css,ts,tsx,jsx,md}`), {
        interval: 0,
    })
        .on('all', (event, p) => __awaiter(void 0, void 0, void 0, function* () {
        if (block && !(0, fsAddons_1.isCss)(p)) {
            return;
        }
        if ((0, fsAddons_1.isJSFile)(p) || (0, fsAddons_1.isTSFile)(p) || (0, fsAddons_1.isMd)(p)) {
            if (fs_1.default.existsSync(p)) {
                block = true;
                yield (0, transform_1.transformFiles)({
                    config,
                });
                if (!liveServerRunning) {
                    liveServerRunning = true;
                    live_server_1.default.start({
                        open: true,
                        logLevel: 0,
                        port: config.port,
                        root: config.out,
                    });
                }
                block = false;
            }
            return;
        }
        (0, transform_1.copyStaticFiles)(config);
    }));
    block = false;
});
exports.watch = watch;
//# sourceMappingURL=serve.js.map