"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateGlobalTypings = exports.envsTypings = exports.basicFunctions = exports.md = void 0;
const graphql_zeus_1 = require("graphql-zeus");
const graphql_js_tree_1 = require("graphql-js-tree");
const dotenv_1 = require("dotenv");
const fs_1 = __importDefault(require("fs"));
const objectToType_1 = require("./objectToType");
exports.md = {
    code: `
import {Remarkable} from 'https://cdn.skypack.dev/remarkable';
const remarkableRenderer = new Remarkable()
export const md = (strings, ...expr) => {
  let str = '';
  strings.forEach((string, i) => {
      str += string + (expr[i] || '');
  });
  return remarkableRenderer.render(str);
}`,
    typings: `export declare const md: (strings: TemplateStringsArray, ...expr: string[]) => string`,
};
exports.basicFunctions = {
    code: `
  export const html =  (strings, ...expr) => {
    let str = '';
    strings.forEach((string, i) => {
        str += string + (expr[i] || '');
    });
    return str;
  }
  export const css =  (strings, ...expr) => {
    let str = '';
    strings.forEach((string, i) => {
        str += string + (expr[i] || '');
    });
    return str;
  }
`,
    typings: `export declare const html: (strings: TemplateStringsArray, ...expr: string[]) => string
export declare const css: (strings: TemplateStringsArray, ...expr: string[]) => string`,
};
const envsTypings = (config) => {
    const envs = fs_1.default.existsSync('./.env')
        ? (0, dotenv_1.parse)(fs_1.default.readFileSync('./.env'))
        : undefined;
    const ssg = {
        envs,
        config,
    };
    return `
declare const ssg: ${(0, objectToType_1.objectToType)(ssg)}`;
};
exports.envsTypings = envsTypings;
const GenerateGlobalTypings = ({ schema, config, }) => {
    const graphqlTree = graphql_js_tree_1.Parser.parse(schema);
    return graphql_zeus_1.TreeToTS.resolveTree({ tree: graphqlTree, env: 'browser' });
};
exports.GenerateGlobalTypings = GenerateGlobalTypings;
//# sourceMappingURL=fn.js.map