"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transpileTS = exports.cleanBuild = exports.copyStaticFiles = exports.copyFile = exports.transformFiles = exports.generateTypingsFiles = exports.generateBasicTypingsFiles = exports.injectHtmlFile = exports.hasTwinFile = exports.createTwinFile = exports.readFiles = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const module_1 = require("./module");
const fn_1 = require("./fn");
const console_1 = require("../console");
const typeFetcher_1 = require("../typeFetcher");
const gray_matter_1 = __importDefault(require("gray-matter"));
const fsAddons_1 = require("../fsAddons");
const esbuild_1 = require("esbuild");
const paths_1 = require("../paths");
const getFiles = (dir) => {
    const result = [];
    const files = [dir];
    do {
        const filepath = files.pop();
        const stat = fs_1.default.lstatSync(filepath);
        if (stat.isDirectory()) {
            fs_1.default.readdirSync(filepath).forEach((f) => files.push(path_1.default.join(filepath, f)));
        }
        else if (stat.isFile()) {
            result.push(path_1.default.relative(dir, filepath));
        }
    } while (files.length !== 0);
    return result;
};
const readFiles = (matchFunction) => (p) => __awaiter(void 0, void 0, void 0, function* () {
    var e_1, _a;
    const allFiles = [];
    try {
        for (var _b = __asyncValues(getFiles(p)), _c; _c = yield _b.next(), !_c.done;) {
            const f = _c.value;
            const t = f;
            if (matchFunction(t)) {
                allFiles.push(t);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return allFiles;
});
exports.readFiles = readFiles;
const createTwinFile = (fileToTransform, extension) => {
    const regexResult = fileToTransform.match(fsAddons_1.fileRegex);
    if (!regexResult) {
        throw new Error('Invalid file provided to function. Only accepting files matching mock regex');
    }
    const twinFileName = extension
        ? `${regexResult[1]}.${extension}`
        : regexResult[1];
    return twinFileName;
};
exports.createTwinFile = createTwinFile;
const hasTwinFile = (fileToTransform, config, extension) => {
    let twinFile;
    const twinFileName = (0, exports.createTwinFile)(fileToTransform, extension);
    const hasCorrespondingTwinFile = fs_1.default.existsSync((0, paths_1.pathIn)(config)(twinFileName));
    if (hasCorrespondingTwinFile) {
        twinFile = twinFileName;
    }
    return twinFile;
};
exports.hasTwinFile = hasTwinFile;
const injectHtmlFile = ({ fileToTransform, config, }) => __awaiter(void 0, void 0, void 0, function* () {
    const cssFile = (0, exports.hasTwinFile)(fileToTransform, config, 'css');
    return {
        name: (0, exports.createTwinFile)(fileToTransform),
        code: yield (0, module_1.bundle)({
            name: fileToTransform,
            config: config,
            css: cssFile,
        }),
    };
});
exports.injectHtmlFile = injectHtmlFile;
const generateBasicTypingsFiles = (config) => __awaiter(void 0, void 0, void 0, function* () {
    const ssgPath = (0, paths_1.pathSsg)(config);
    (0, fsAddons_1.fileWriteRecuirsiveSync)(ssgPath('md.js'), fn_1.md.code);
    (0, fsAddons_1.fileWriteRecuirsiveSync)(ssgPath('md.d.ts'), fn_1.md.typings);
    (0, fsAddons_1.fileWriteRecuirsiveSync)(ssgPath('basic.js'), fn_1.basicFunctions.code);
    (0, fsAddons_1.fileWriteRecuirsiveSync)(ssgPath('basic.d.ts'), fn_1.basicFunctions.typings);
    (0, fsAddons_1.fileWriteRecuirsiveSync)((0, paths_1.pathIn)(config)('purplehaze.d.ts'), (0, fn_1.envsTypings)(config));
});
exports.generateBasicTypingsFiles = generateBasicTypingsFiles;
const generateTypingsFiles = ({ name, schema, config, }) => __awaiter(void 0, void 0, void 0, function* () {
    const typings = yield (0, fn_1.GenerateGlobalTypings)({
        config,
        schema,
    });
    const ssgPath = (0, paths_1.pathSsg)(config);
    (0, fsAddons_1.fileWriteRecuirsiveSync)(ssgPath('index.ts'), typings);
});
exports.generateTypingsFiles = generateTypingsFiles;
const transformFiles = ({ config }) => __awaiter(void 0, void 0, void 0, function* () {
    const ssgPath = (0, paths_1.pathSsg)(config);
    const mdFiles = yield (0, exports.readFiles)(fsAddons_1.isMd)(config.in);
    if (mdFiles.length > 0) {
        const generatedMdLib = {};
        try {
            yield Promise.all(mdFiles.map((mdFile) => __awaiter(void 0, void 0, void 0, function* () {
                const m = (0, gray_matter_1.default)(fs_1.default.readFileSync((0, paths_1.pathIn)(config)(mdFile)));
                generatedMdLib[mdFile] = {
                    content: m.content,
                    data: m.data,
                    excerpt: m.excerpt,
                };
            })));
        }
        catch (error) {
            if (error instanceof Error) {
                (0, console_1.message)(error.message, 'red');
            }
            return;
        }
        (0, fsAddons_1.fileWriteRecuirsiveSync)(ssgPath('markdown.ts'), `export const htmlContent = ${JSON.stringify(generatedMdLib, null, 4)} as const`);
    }
    const tsFiles = yield (0, exports.readFiles)(fsAddons_1.isTSFile)(config.in);
    if (tsFiles.length > 0) {
        const tsconfig = JSON.parse(fs_1.default.readFileSync('./tsconfig.json').toString('utf-8'));
        try {
            yield Promise.all(tsFiles.map((tsFile) => __awaiter(void 0, void 0, void 0, function* () {
                const transpiled = yield (0, exports.transpileTS)(fs_1.default.readFileSync((0, paths_1.pathIn)(config)(tsFile)).toString('utf-8'), tsFile.endsWith('tsx') ? 'tsx' : 'ts', tsconfig);
                const jsFileName = (0, paths_1.pathIn)(config)(tsFile.replace(/\.tsx?$/, '.js'));
                (0, fsAddons_1.fileWriteRecuirsiveSync)(jsFileName, transpiled.code);
            })));
        }
        catch (error) {
            if (error instanceof Error) {
                (0, console_1.message)(error.message, 'red');
            }
            return;
        }
    }
    const files = yield (0, exports.readFiles)(fsAddons_1.isJSFile)(config.in);
    (0, console_1.message)('Sending code to browser', 'yellowBright');
    const htmlFiles = (yield Promise.all(files.map((f) => (0, exports.injectHtmlFile)({ fileToTransform: f, config })))).filter((f) => f.code);
    (0, console_1.message)('Code render successful', 'greenBright');
    const rf = files.map((f) => ({
        name: f,
        path: (0, paths_1.pathOut)(config)(f),
        content: fs_1.default.readFileSync((0, paths_1.pathIn)(config)(f)),
    }));
    yield (0, typeFetcher_1.downloadTypings)(config, rf.map((r) => r.content.toString('utf-8')));
    (0, console_1.message)(`Writing out ${rf.length} files.`, 'yellow');
    rf.forEach((f) => {
        (0, fsAddons_1.fileWriteRecuirsiveSync)((0, paths_1.pathOut)(config)(f.name), f.content);
    });
    (0, console_1.message)(`Writing out ${htmlFiles.length} pages.`, 'yellow');
    htmlFiles.forEach(({ name, code }) => {
        if (code === null || code === void 0 ? void 0 : code.pages) {
            code.pages.forEach((page) => {
                (0, fsAddons_1.fileWriteRecuirsiveSync)((0, paths_1.pathOut)(config)(name, page.slug + '.html'), page.body);
            });
            return;
        }
        if (code === null || code === void 0 ? void 0 : code.content) {
            (0, fsAddons_1.fileWriteRecuirsiveSync)(`${(0, paths_1.pathOut)(config)(name)}.html`, code.content);
        }
    });
});
exports.transformFiles = transformFiles;
const copyFile = (config) => (relativeFilePath) => {
    (0, fsAddons_1.fileWriteRecuirsiveSync)((0, paths_1.pathOut)(config)(relativeFilePath), fs_1.default.readFileSync((0, paths_1.pathIn)(config)(relativeFilePath)));
};
exports.copyFile = copyFile;
const copyStaticFiles = (config) => {
    const files = getFiles(config.in);
    files
        .filter((f) => !(0, fsAddons_1.isDirectory)((0, paths_1.pathIn)(config)(f)))
        .filter(fsAddons_1.isStaticFile)
        .forEach((f) => {
        (0, exports.copyFile)(config)(f);
    });
};
exports.copyStaticFiles = copyStaticFiles;
const cleanBuild = (config) => {
    if (fs_1.default.existsSync(config.out)) {
        fs_1.default.rmSync(config.out, { recursive: true });
    }
};
exports.cleanBuild = cleanBuild;
const transpileTS = (code, loader, options) => {
    return (0, esbuild_1.transform)(code, {
        tsconfigRaw: JSON.stringify(options),
        loader,
        format: 'esm',
    });
};
exports.transpileTS = transpileTS;
//# sourceMappingURL=index.js.map