"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateJSConfig = exports.updateTSConfig = exports.regenerateJsConfig = exports.regenerateTsConfig = exports.getTsConfig = exports.getJsConfig = exports.JSConfig = exports.TSConfig = void 0;
const fsAddons_1 = require("./fsAddons");
const fs_1 = __importDefault(require("fs"));
const TSConfig = () => ({
    compilerOptions: {
        incremental: true,
        target: 'esnext',
        module: 'ES2020',
        strict: true,
        esModuleInterop: true,
        skipLibCheck: true,
        forceConsistentCasingInFileNames: true,
        allowJs: true,
        noEmit: true,
        baseUrl: '',
    },
});
exports.TSConfig = TSConfig;
const JSConfig = () => ({
    compilerOptions: {
        target: 'esnext',
        module: 'commonjs',
        baseUrl: './',
    },
    include: ['./pages/**/*', './pages/purplehaze.d.ts'],
});
exports.JSConfig = JSConfig;
const getJsConfig = () => (0, fsAddons_1.existsJSONOrDefaultSync)('./jsconfig.json', (0, exports.JSConfig)());
exports.getJsConfig = getJsConfig;
const getTsConfig = (config) => (0, fsAddons_1.existsJSONOrDefaultSync)('./tsconfig.json', (0, exports.TSConfig)());
exports.getTsConfig = getTsConfig;
const regenerateTsConfig = (config) => {
    (0, exports.updateTSConfig)(config, (oldConfig) => {
        return Object.assign(Object.assign({}, oldConfig), { include: [
                `${config.in}/**/*.tsx`,
                `${config.in}/**/*.jsx`,
                `${config.in}/**/*.ts`,
                `${config.in}/**/*.js`,
                `${config.in}/purplehaze.d.ts`,
            ] });
    });
};
exports.regenerateTsConfig = regenerateTsConfig;
const regenerateJsConfig = (config) => {
    (0, exports.updateJSConfig)((oldConfig) => {
        return Object.assign(Object.assign({}, oldConfig), { include: [`${config.in}/**/*.js`, `${config.in}/purplehaze.d.ts`] });
    });
};
exports.regenerateJsConfig = regenerateJsConfig;
const updateTSConfig = (config, fn) => {
    fs_1.default.writeFileSync('./tsconfig.json', JSON.stringify(fn((0, exports.getTsConfig)(config)), null, 2));
};
exports.updateTSConfig = updateTSConfig;
const updateJSConfig = (fn) => {
    fs_1.default.writeFileSync('./jsconfig.json', JSON.stringify(fn((0, exports.getJsConfig)()), null, 2));
};
exports.updateJSConfig = updateJSConfig;
//# sourceMappingURL=transpilerConfig.js.map